﻿Imports Infragistics.Win.DataVisualization
Imports System.Collections.ObjectModel
Imports Infragistics.Win.UltraWinSchedule

Public Class ContactVEventsChart
    Private WithEvents _ultraDataChart As UltraDataChart
    Private _ultraLegend As UltraLegend
    Private WithEvents _fromDate As UltraCalendarCombo
    Private WithEvents _toDate As UltraCalendarCombo


    Public Sub New(ultraDataChart As UltraDataChart, ultraLegend As UltraLegend,
                   fromDate As UltraCalendarCombo, toDate As UltraCalendarCombo)
        _ultraDataChart = ultraDataChart
        _ultraLegend = ultraLegend
        _fromDate = fromDate
        _toDate = toDate
    End Sub



    Public Sub LoadData()
        _ultraDataChart.Axes.Clear()
        _ultraDataChart.Series.Clear()

        'Dim startDate As Date = Date.Parse("9 Jun 2014")
        'Dim endDate As Date = Date.Parse("30 Nov 2014")

        Dim startDate As Date = _fromDate.Value
        Dim endDate As Date = _toDate.Value
        Dim daysTimeSpan As TimeSpan = endDate.Subtract(startDate)
        Dim dayCount As Integer = daysTimeSpan.Days


        'The series must have the same number of objects (dates) as each other
        'so the data set returns all the days within the range even if
        'there are no counts on those days
        Dim posContacts As List(Of DashboardContactsVEventsContactDataIC) = GetPositiveContactsList()

        Dim negContacts As List(Of DashboardContactsVEventsContactDataIC) = GetNegativeContactsList()

        'Dim a As DashboardContactsVEventsContactDataIC
        'a.ActivityDate
        'a.ContactCount()

        Dim posScale As Integer
        If posContacts.Count > 0 Then
            posScale = posContacts.Max(Function(s) (s.ContactCount))
        Else
            posScale = 5
        End If

        Dim negScale As Integer
        If negContacts.Count > 0 Then
            negScale = negContacts.Min(Function(s) (s.ContactCount))
        Else
            negScale = -2
        End If



        Dim categoryX As CategoryDateTimeXAxis = New CategoryDateTimeXAxis()
        categoryX.DateTimeMemberPath = "ActivityDate"
        categoryX.DisplayType = TimeAxisDisplayType.Continuous
        categoryX.LabelExtent = 50
        categoryX.LabelsVisible = True
        categoryX.LabelTextColor = Color.Gray
        categoryX.LabelTextStyle = FontStyle.Regular
        categoryX.LabelFontFamily = "Arial"
        categoryX.LabelFontSize = 9
        categoryX.LabelLocation = AxisLabelsLocation.OutsideBottom

        'interval is dependant upon the date range.  We are only displaying 12 dates so get the
        'no of days between the two dates then divide by 10.  Round up and use that is the interval
        Dim interval As Integer = Math.Round(dayCount / 10, System.MidpointRounding.AwayFromZero)


        categoryX.Interval = New TimeSpan(Math.Max(interval, 1), 0, 0, 0)
        categoryX.Label = "ActivityDate"
        categoryX.DataSource = posContacts
        categoryX.MajorStroke = Brushes.Transparent
        categoryX.MinorStroke = Brushes.Transparent
        categoryX.Stroke = Brushes.Gray
        categoryX.StrokeThickness = 2

        Dim categoryXevents As CategoryXAxis = New CategoryXAxis()
        Dim numList = Enumerable.Range(0, Math.Max(posContacts.Count, 1) - 1).ToList()
        categoryXevents.DataSource = numList


        Dim numericY As NumericYAxis = New NumericYAxis()
        numericY.MaximumValue = Math.Max(posScale, 5)
        numericY.MinimumValue = Math.Min(negScale, -2)


        numericY.MajorStroke = Brushes.Transparent
        numericY.MinorStroke = Brushes.Transparent
        numericY.Stroke = Brushes.Black
        numericY.StrokeThickness = 2


        categoryX.CrossingAxis = numericY
        categoryX.CrossingValue = 0
        categoryX.Title = "Date"
        'numericY.Title = "Count"
        numericY.LabelExtent = 60
        numericY.LabelFontSize = 10
        numericY.LabelFontFamily = "Arial"
        'numericY.Interval = 1



        _ultraDataChart.Axes.Add(categoryX)
        _ultraDataChart.Axes.Add(categoryXevents)
        _ultraDataChart.Axes.Add(numericY)

        Dim lineSeries As LineSeries = New LineSeries()
        lineSeries.Title = "Contacts Added"
        lineSeries.XAxis = categoryX
        lineSeries.YAxis = numericY
        lineSeries.ValueMemberPath = "ContactCount"
        lineSeries.DataSource = posContacts 'testData
        lineSeries.Thickness = 1.5
        lineSeries.Brush = New Infragistics.Win.DataVisualization.SolidColorBrush(Color.LimeGreen)

        Dim lineSeries2 As LineSeries = New LineSeries()
        lineSeries2.Title = "Contacts Deleted"
        lineSeries2.XAxis = categoryX
        lineSeries2.YAxis = numericY
        lineSeries2.ValueMemberPath = "ContactCount"
        lineSeries2.DataSource = negContacts
        lineSeries2.Thickness = 1.5
        lineSeries2.Brush = New Infragistics.Win.DataVisualization.SolidColorBrush(Color.Red)

        _ultraDataChart.Series.Add(lineSeries)
        _ultraDataChart.Series.Add(lineSeries2)

        _ultraDataChart.HorizontalZoomable = True
        _ultraDataChart.VerticalZoomable = True

        '_ultraDataChart.WindowResponse = WindowResponse.Immediate
        '_ultraDataChart.Dock = DockStyle.Fill
        'Controls.Add(_ultraDataChart)
        _ultraDataChart.Show()


        _ultraDataChart.Legend = _ultraLegend

        Dim events As List(Of MarketingCalendarDataIC) = _
            GetMarketingCalendarData()

        For Each e As MarketingCalendarDataIC In events
            'we need to get the index of the date in either the pos or neg contacts list
            'this determines the X position on the scatter graph we're using for 
            'plotting the vertical line

            Dim line As DataChartLine = New DataChartLine(_ultraDataChart,
                    GetPositionInListBasedOnDate(posContacts, e.StartDate.Date), e.EventTypeDescription, Color.FromName(e.CalendarColour))

            lines.Add(line)
        Next

        SetAxisHandler()
    End Sub

    Private Sub ClearAxisHandlers()
        Dim catX As CategoryDateTimeXAxis = GetDateAxis()

        If catX IsNot Nothing Then
            RemoveHandler catX.FormatLabel, AddressOf OnAxisXFormatLabel
        End If

    End Sub
    Private Function GetDateAxis() As CategoryDateTimeXAxis
        Return _ultraDataChart.Axes.OfType(Of Infragistics.Win.DataVisualization.CategoryDateTimeXAxis).FirstOrDefault()
    End Function
    Private Sub SetAxisHandler()
        ClearAxisHandlers()

        Dim catX As CategoryDateTimeXAxis = GetDateAxis()
        If catX IsNot Nothing Then
            AddHandler catX.FormatLabel, AddressOf OnAxisXFormatLabel
        End If
    End Sub

    Private Function GetPositionInListBasedOnDate(contactStats As List(Of DashboardContactsVEventsContactDataIC), startDate As DateTime) As Integer
        Dim item = contactStats.SingleOrDefault(Function(c) (c.ActivityDate = startDate))

        Return contactStats.IndexOf(item)
    End Function

    Private Shared Function GetContactData(ByVal items As List(Of ContactVEventsContactChartData)) As DataTable
        Dim table As New DataTable()
        table.Columns.Add("Label", GetType(String))
        table.Columns.Add("Count", GetType(Integer))
        table.Columns.Add("ActivityDate", GetType(DateTime))
        For Each i In items
            table.Rows.Add(New Object() {i.Label, i.Count, i.ActivityDate})
        Next
        Return table
    End Function

    Private Shared Function GetData() As DataTable
        Dim table As New DataTable()
        table.Columns.Add("Label Column", GetType(String))
        table.Columns.Add("Value Column", GetType(Double))
        table.Columns.Add("Another Value Column", GetType(Double))
        table.Rows.Add(New Object() {"Point A", 1.0, 3.0})
        table.Rows.Add(New Object() {"Point B", 2.0, 2.0})
        table.Rows.Add(New Object() {"Point C", 3.0, 1.0})
        table.Rows.Add(New Object() {"Point D", 4.0, 2.0})
        table.Rows.Add(New Object() {"Point E", 5.0, 3.0})
        Return table
    End Function


    Private Class ContactVEventsContactChartData
        Public Property Label As String
        Public Property Count As Integer
        'Public Property Test As String
        Public Property ActivityDate As DateTime
    End Class

    Public Class TestData
        Inherits ObservableCollection(Of Double)

        Private Shared _rand As Random = New Random()

        Public Sub New(scaleFactor As Double)
            For i As Integer = 0 To 10
                Add(_rand.NextDouble() * scaleFactor)
            Next

        End Sub
    End Class


    Private lines As List(Of DataChartLine) = New List(Of DataChartLine)

    Private Sub _ultraDataChart_RefreshCompleted(sender As Object, e As EventArgs) Handles _ultraDataChart.RefreshCompleted
        Dim catX As CategoryXAxis = _ultraDataChart.Axes.OfType(Of Infragistics.Win.DataVisualization.CategoryXAxis).FirstOrDefault()
        Dim numX As NumericXAxis = _ultraDataChart.Axes.OfType(Of Infragistics.Win.DataVisualization.NumericXAxis).FirstOrDefault()

        If catX Is Nothing OrElse numX Is Nothing Then
            Exit Sub
        End If

        For Each line As DataChartLine In lines
            ' Find the X coordinate of our target point
            Dim pixelValue As Double = catX.ScaleValue(line.TargetValue)

            ' If the chart is using a clustering series, we need to offset the line a bit
            Dim clusterMode As Boolean = False
            If (_ultraDataChart.Series.Where(Function(s) _
                    (TypeOf s Is Infragistics.Win.DataVisualization.ColumnSeries OrElse _
                    TypeOf s Is Infragistics.Win.DataVisualization.WaterfallSeries OrElse _
                    TypeOf s Is Infragistics.Win.DataVisualization.RangeColumnSeries)).Any()) Then

                clusterMode = True
            End If


            If (clusterMode) Then

                pixelValue += _ultraDataChart.Series.OfType(Of Infragistics.Win.DataVisualization.CategorySeries).First() _
                        .GetCategoryWidth() / 2.0
            End If

            ' Find the value of our target point on the numeric axis
            Dim axisValue As Double = numX.UnscaleValue(pixelValue)
            line.UpdateLine(axisValue)

        Next
    End Sub

    Private Sub _fromDate_TextChanged(sender As Object, e As EventArgs) Handles _fromDate.TextChanged
        LoadData()
    End Sub

    Private Sub _toDate_TextChanged(sender As Object, e As EventArgs) Handles _toDate.TextChanged
        LoadData()
    End Sub

#Region "OnAxisXFormatLabel"
    Private Function OnAxisXFormatLabel(info As AxisLabelInfo) As String
        Return info.DateValue.ToString("dd/MM/yyyy")
    End Function
#End Region

    Private Function GetPositiveContactsList() As List(Of DashboardContactsVEventsContactDataIC)
        Dim l As New List(Of DashboardContactsVEventsContactDataIC)

        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-10), .ContactCount = 5})
        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-9), .ContactCount = 0})
        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-8), .ContactCount = 1})
        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-7), .ContactCount = 6})
        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-6), .ContactCount = 0})
        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-5), .ContactCount = 9})

        Return l

    End Function

    Private Function GetNegativeContactsList() As List(Of DashboardContactsVEventsContactDataIC)
        Dim l As New List(Of DashboardContactsVEventsContactDataIC)

        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-10), .ContactCount = 0})
        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-9), .ContactCount = -1})
        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-8), .ContactCount = 0})
        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-7), .ContactCount = 0})
        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-6), .ContactCount = -2})
        l.Add(New DashboardContactsVEventsContactDataIC With {.ActivityDate = Now.AddDays(-5), .ContactCount = -1})

        Return l
    End Function

    Private Function GetMarketingCalendarData() As List(Of MarketingCalendarDataIC)
        Dim l As New List(Of MarketingCalendarDataIC)

        l.Add(New MarketingCalendarDataIC With {
                .ItemId = 1, .TypeId = 1, .EventTypeDescription = "Advert",
                .StartDate = Now.AddDays(-9), .EndDate = Now.AddDays(-9), .AllDayEvent = False,
                .HasAttachments = 0, .CalendarColour = "Orange"})
        l.Add(New MarketingCalendarDataIC With {
          .ItemId = 1, .TypeId = 2, .EventTypeDescription = "Share Sale",
          .StartDate = Now.AddDays(-6), .EndDate = Now.AddDays(-6), .AllDayEvent = False,
          .HasAttachments = 0, .CalendarColour = "Blue"})

        Return l

    End Function

End Class
