﻿Imports Infragistics.Win.DataVisualization
Imports System.Collections.ObjectModel
Imports DataChartGap.DataGridVerticalGridlines

Public Class DataChartLine
    Private annotationCollection As ObservableCollection(Of AnnotationPoint)


    Private _targetValue As Double
    Private P0 As AnnotationPoint
    Private P1 As AnnotationPoint
    Private lastAxisValue As Double = -1

    Public ReadOnly Property TargetValue As Double
        Get
            Return _targetValue
        End Get
    End Property


    Public Sub New(chart As UltraDataChart, targetVal As Double, title As String, color As Color)
        ' Create a Y axis for the line, so that we can easily control the length
        Dim numericY2 As NumericYAxis = New NumericYAxis()
        numericY2.MinimumValue = 0
        numericY2.MaximumValue = 1
        numericY2.LabelsVisible = False

        numericY2.MajorStroke = Brushes.Transparent
        numericY2.MinorStroke = Brushes.Transparent
        numericY2.Stroke = Brushes.Transparent
        'numericY2.StrokeThickness = 5

        ' Create an X axis for the line, to easily control the position
        Dim numericX2 As NumericXAxis = New NumericXAxis()
        numericX2.MinimumValue = 0
        numericX2.MaximumValue = 1
        numericX2.LabelsVisible = False
        numericX2.LabelExtent = 0

        numericX2.MajorStroke = Brushes.Transparent
        numericX2.MinorStroke = Brushes.Transparent
        numericX2.Stroke = Brushes.Transparent
        'numericX2.StrokeThickness = 5

        chart.Axes.Add(numericX2)
        chart.Axes.Add(numericY2)

        ' A scatterline will allow us to specify coordinates for the start and end of the line we'll draw
        Dim scatterLine As ScatterLineSeries = New ScatterLineSeries()
        scatterLine.Brush = New SolidColorBrush(color)
        scatterLine.Thickness = 2.0
        scatterLine.XAxis = numericX2
        scatterLine.YAxis = numericY2

        ' Set up the points
        annotationCollection = New ObservableCollection(Of AnnotationPoint)
        P0 = New AnnotationPoint()
        P1 = New AnnotationPoint()
        annotationCollection.Add(P0)
        annotationCollection.Add(P1)

        scatterLine.Title = title
        _targetValue = targetVal

        scatterLine.DataSource = annotationCollection
        scatterLine.XMemberPath = "X"
        scatterLine.YMemberPath = "Y"

        For Each s As Series In chart.Series
            If TypeOf s Is ScatterLineSeries Then
                If title = s.Title Then
                    scatterLine.LegendItemVisibility = Infragistics.Portable.Components.UI.Visibility.Collapsed
                    Exit For

                End If
            End If
        Next


        chart.Series.Add(scatterLine)
    End Sub

    Public Sub UpdateLine(axisValue As Double)

        ' If the point is changed, update its location
        If axisValue <> lastAxisValue Then
            lastAxisValue = axisValue

            P0.X = axisValue
            P1.X = axisValue
            P0.Y = 0
            P1.Y = 1.0
        End If

    End Sub



End Class
